// jQuery File Tree Plugin
//
// Version 1.01
//
// Cory S.N. LaViska
// A Beautiful Site (http://abeautifulsite.net/)
// 24 March 2008
//
// Visit http://abeautifulsite.net/notebook.php?article=58 for more information
//
// Usage: $('.fileTreeDemo').fileTree( options, callback )
//
// Options:  root           - root folder to display; default = /
//           script         - location of the serverside AJAX file to use; default = jqueryFileTree.php
//           folderEvent    - event to trigger expand/collapse; default = click
//           expandSpeed    - default = 500 (ms); use -1 for no animation
//           collapseSpeed  - default = 500 (ms); use -1 for no animation
//           expandEasing   - easing function to use on expand (optional)
//           collapseEasing - easing function to use on collapse (optional)
//           multiFolder    - whether or not to limit the browser to one subfolder at a time
//           loadMessage    - Message to display while initial tree loads (can be HTML)
//
// History:
//
// 1.01 - updated to work with foreign characters in directory/file names (12 April 2008)
// 1.00 - released (24 March 2008)
//
// TERMS OF USE
// 
// This plugin is dual-licensed under the GNU General Public License and the MIT License and
// is copyright 2008 A Beautiful Site, LLC. 
//
if(jQuery) (function($){
	
	$.extend($.fn, {
		ldapTree: function(o, h, hh, move) {
			// Defaults
			if( !o ) var o = {};
			if( o.root == undefined ) o.root = '/';
			if( o.script == undefined ) o.script = 'jqueryFileTree.php';
			if( o.folderEvent == undefined ) o.folderEvent = 'click';
			if( o.fileEvent == undefined ) o.fileEvent = 'click';
			if( o.expandSpeed == undefined ) o.expandSpeed= 500;
			if( o.collapseSpeed == undefined ) o.collapseSpeed= 500;
			if( o.expandEasing == undefined ) o.expandEasing = null;
			if( o.collapseEasing == undefined ) o.collapseEasing = null;
			if( o.multiFolder == undefined ) o.multiFolder = true;
			if( o.loadMessage == undefined ) o.loadMessage = 'Loading...';
			
			$(this).each( function() {
				
				function showTree(c, t) {
					$(c).addClass('wait');
					$(".jqueryLdapTree.start").remove();
					$.post(o.script, {dir: t}, function(data) {
						$(c).find('.start').html('');
						if(data == 'NULL')
						{
							$(c).removeClass('wait').addClass('empty');
						}
						else
						{
							$(c).removeClass('wait').append(data);
							if( o.root == t ) $(c).find('UL:hidden').show(); else $(c).find('UL:hidden').slideDown({duration: o.expandSpeed, easing: o.expandEasing});
							bindTree(c);
						}
					});
				}
				
				function bindTree(t) {
					// Для директорий вешаем такую функцию
					$(t).find('LI.directory A').parent().bind(o.folderEvent, function() {
						if( $(this).hasClass('collapsed') ) {
							// Expand
							if( !o.multiFolder ) {
								$(this).parent().find('UL').slideUp({duration: o.collapseSpeed, easing: o.collapseEasing});
								$(this).parent().find('LI.directory').removeClass('expanded').addClass('collapsed');
							}

							$(this).find('UL').remove(); // cleanup
							showTree( $(this), $(this).children('A').attr('rel') );
							$(this).removeClass('collapsed').addClass('expanded');
						} else {
							// Collapse
							$(this).find('UL').slideUp({duration: o.collapseSpeed, easing: o.collapseEasing});
							$(this).removeClass('expanded').addClass('collapsed');
						}
						h($(this).children('A').attr('rel'));
						return false;
					}).droppable({
						greedy: true,
						drop: function(event, ui){
							moveelement = ui.draggable.children('A').attr('rel');
							movetarget = $(this).children('A').attr('rel');
							move(moveelement, movetarget);
							if($(this).children("UL").length)
								$(this).children("UL").append(ui.draggable);
							else
								ui.draggable.html('');
						}
					});
					// Для не директорий вушаем такую
					$(t).find('LI.file A').parent().bind(o.fileEvent, function() {
						hh($(this).children('A').attr('rel'));
						return false;
					}).draggable({
						revert: true
					});
					// Prevent A from triggering the # on non-click events
					if( o.folderEvent.toLowerCase != 'click' ) $(t).find('LI.directory A').parent().bind('click', function() {return false;});
					if( o.fileEvent.toLowerCase != 'click' ) $(t).find('LI.file A').parent().bind('click', function() {return false;});
				}
				// Loading message
				$(this).html('<ul class="jqueryLdapTree start"><li class="wait">' + o.loadMessage + '<li></ul>');
				// Get the initial file list
				showTree( $(this), o.root );
			});
		}
	});
	
})(jQuery);